r"""
H-trivial semigroups
"""
#*****************************************************************************
#  Copyright (C) 2016 Nicolas M. Thiéry <nthiery at users.sf.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.categories.category_with_axiom import CategoryWithAxiom
from sage.categories.semigroups import Semigroups

class HTrivialSemigroups(CategoryWithAxiom):
    def Finite_extra_super_categories(self):
        r"""
        Implement the fact that a finite `H`-trivial is aperiodic

        EXAMPLES::

            sage: Semigroups().HTrivial().Finite_extra_super_categories()
            [Category of aperiodic semigroups]
            sage: Semigroups().HTrivial().Finite() is Semigroups().Aperiodic().Finite()
            True
        """
        return [Semigroups().Aperiodic()]

    def Inverse_extra_super_categories(self):
        r"""
        Implement the fact that an `H`-trivial inverse semigroup is `J`-trivial.

        .. TODO::

            Generalization for inverse semigroups.

            Recall that there are two invertibility axioms for a semigroup `S`:

            - One stating the existence, for all `x`, of a local inverse
              `y` satisfying `x=xyx` and `y=yxy`;
            - One stating the existence, for all `x`, of a global
              inverse `y` satisfying `xy=yx=1`, where `1` is the unit
              of `S` (which must of course exist).

            It is sufficient to have local inverses for `H`-triviality
            to imply `J`-triviality. However, at this stage, only the
            second axiom is implemented in Sage (see
            :meth:`Magmas.Unital.SubcategoryMethods.Inverse`). Therefore
            this fact is only implemented for semigroups with global
            inverses, that is groups. However the trivial group is the
            unique `H`-trivial group, so this is rather boring.

        EXAMPLES::

            sage: Semigroups().HTrivial().Inverse_extra_super_categories()
            [Category of j trivial semigroups]
            sage: Monoids().HTrivial().Inverse()
            Category of h trivial groups
        """
        return [self.JTrivial()]



r"""
Pointed sets
"""
#*****************************************************************************
#  Copyright (C) 2008 David Kohel <kohel@maths.usyd.edu> and
#                     William Stein <wstein@math.ucsd.edu>
#                     Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.categories.category_singleton import Category_singleton
from sage.categories.sets_cat import Sets


class PointedSets(Category_singleton):
    """
    The category of pointed sets.

    EXAMPLES::

        sage: PointedSets()
        Category of pointed sets

    TESTS::

        sage: TestSuite(PointedSets()).run()
    """
    #def __call__(self, X, pt):
    #    import sage.sets.all
    #    return sage.sets.all.Set(X, pt)

    def super_categories(self):
        """
        EXAMPLES::

            sage: PointedSets().super_categories()
            [Category of sets]
        """
        return [Sets()] # ???



r"""
Monoids
"""
# ****************************************************************************
#  Copyright (C) 2005      David Kohel <kohel@maths.usyd.edu>
#                          William Stein <wstein@math.ucsd.edu>
#                2008      Teresa Gomez-Diaz (CNRS) <Teresa.Gomez-Diaz@univ-mlv.fr>
#                2008-2009 Florent Hivert <florent.hivert at univ-rouen.fr>
#                2008-2014 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from sage.misc.cachefunc import cached_method
from sage.categories.category_with_axiom import CategoryWithAxiom
from sage.categories.semigroups import Semigroups
from sage.misc.lazy_import import LazyImport
from sage.categories.subquotients import SubquotientsCategory
from sage.categories.cartesian_product import CartesianProductsCategory
from sage.categories.algebra_functor import AlgebrasCategory
from sage.categories.with_realizations import WithRealizationsCategory
from sage.categories.finite_enumerated_sets import FiniteEnumeratedSets
from sage.arith.power import generic_power


class Monoids(CategoryWithAxiom):
    r"""
    The category of (multiplicative) monoids.

    A *monoid* is a unital :class:`semigroup <Semigroups>`, that is a
    set endowed with a multiplicative binary operation `*` which is
    associative and admits a unit (see :wikipedia:`Monoid`).

    EXAMPLES::

        sage: Monoids()
        Category of monoids
        sage: Monoids().super_categories()
        [Category of semigroups, Category of unital magmas]
        sage: Monoids().all_super_categories()
        [Category of monoids,
         Category of semigroups,
         Category of unital magmas, Category of magmas,
         Category of sets,
         Category of sets with partial maps,
         Category of objects]

        sage: Monoids().axioms()
        frozenset({'Associative', 'Unital'})
        sage: Semigroups().Unital()
        Category of monoids

        sage: Monoids().example()
        An example of a monoid: the free monoid generated by ('a', 'b', 'c', 'd')

    TESTS::

        sage: C = Monoids()
        sage: TestSuite(C).run()

    ::

        sage: S = Monoids().example()
        sage: x = S("aa")
        sage: x^0, x^1, x^2, x^3, x^4, x^5
        ('', 'aa', 'aaaa', 'aaaaaa', 'aaaaaaaa', 'aaaaaaaaaa')

    Check for :trac:`31212`::

        sage: R = IntegerModRing(15)
        sage: R.submonoid([R.one()]).list()                                             # needs sage.combinat
        [1]
    """
    _base_category_class_and_axiom = (Semigroups, "Unital")

    Finite = LazyImport('sage.categories.finite_monoids', 'FiniteMonoids', at_startup=True)
    Inverse = LazyImport('sage.categories.groups', 'Groups', at_startup=True)

    @staticmethod
    def free(index_set=None, names=None, **kwds):
        r"""
        Return a free monoid on `n` generators or with the generators
        indexed by a set `I`.

        A free monoid is constructed by specifying either:

        - the number of generators and/or the names of the generators
        - the indexing set for the generators

        INPUT:

        - ``index_set`` -- (optional) an index set for the generators; if
          an integer, then this represents `\{0, 1, \ldots, n-1\}`

        - ``names`` -- a string or list/tuple/iterable of strings
          (default: ``'x'``); the generator names or name prefix

        EXAMPLES::

            sage: Monoids.free(index_set=ZZ)                                            # needs sage.combinat
            Free monoid indexed by Integer Ring
            sage: Monoids().free(ZZ)                                                    # needs sage.combinat
            Free monoid indexed by Integer Ring
            sage: F.<x,y,z> = Monoids().free(); F                                       # needs sage.combinat
            Free monoid indexed by {'x', 'y', 'z'}
        """
        if names is not None:
            if isinstance(names, str):
                from sage.rings.integer_ring import ZZ
                if ',' not in names and index_set in ZZ:
                    names = [names + repr(i) for i in range(index_set)]
                else:
                    names = names.split(',')
            names = tuple(names)
            if index_set is None:
                index_set = names

        from sage.monoids.indexed_free_monoid import IndexedFreeMonoid
        return IndexedFreeMonoid(index_set, names=names, **kwds)

    class ParentMethods:

        def semigroup_generators(self):
            """
            Return the generators of ``self`` as a semigroup.

            The generators of a monoid `M` as a semigroup are the generators
            of `M` as a monoid and the unit.

            EXAMPLES::

                sage: M = Monoids().free([1,2,3])                                       # needs sage.combinat
                sage: M.semigroup_generators()                                          # needs sage.combinat
                Family (1, F[1], F[2], F[3])
            """
            G = self.monoid_generators()
            from sage.categories.finite_enumerated_sets import FiniteEnumeratedSets
            if G not in FiniteEnumeratedSets():
                raise NotImplementedError("currently only implemented for finitely generated monoids")
            from sage.sets.family import Family
            return Family((self.one(),) + tuple(G))

        def prod(self, args):
            r"""
            n-ary product of elements of ``self``.

            INPUT:

            - ``args`` -- a list (or iterable) of elements of ``self``

            Returns the product of the elements in ``args``, as an element of
            ``self``.

            EXAMPLES::

                sage: S = Monoids().example()
                sage: S.prod([S('a'), S('b')])
                'ab'
            """
            from sage.misc.misc_c import prod
            return prod(args, self.one())

        def _test_prod(self, **options):
            r"""
            Run basic tests for the product method :meth:`prod` of ``self``.

            See the documentation for :class:`TestSuite` for information on
            further options.

            INPUT:

            - ``options`` -- any keyword arguments accepted by :meth:`_tester`

            EXAMPLES:

            By default, this method tests only the elements returned by
            ``self.some_elements()``::

                sage: S = Monoids().example()
                sage: S._test_prod()

            However, the elements tested can be customized with the
            ``elements`` keyword argument::

                sage: S._test_prod(elements = (S('a'), S('b')))
            """
            tester = self._tester(**options)
            tester.assertEqual(self.prod([]), self.one())
            for x in tester.some_elements():
                tester.assertEqual(self.prod([x]), x)
                tester.assertEqual(self.prod([x, x]), x**2)
                tester.assertEqual(self.prod([x, x, x]), x**3)

        def submonoid(self, generators, category=None):
            r"""
            Return the multiplicative submonoid generated by ``generators``.

            INPUT:

            - ``generators`` -- a finite family of elements of
              ``self``, or a list, iterable, ... that can be converted
              into one (see :class:`Family`).

            - ``category`` -- a category

            This is a shorthand for
            :meth:`Semigroups.ParentMethods.subsemigroup` that
            specifies that this is a submonoid, and in particular that
            the unit is ``self.one()``.

            EXAMPLES::

                sage: R = IntegerModRing(15)
                sage: M = R.submonoid([R(3), R(5)]); M                                  # needs sage.combinat
                A submonoid of (Ring of integers modulo 15) with 2 generators
                sage: M.list()                                                          # needs sage.combinat
                [1, 3, 5, 9, 0, 10, 12, 6]

            Not the presence of the unit, unlike in::

                sage: S = R.subsemigroup([R(3), R(5)]); S                               # needs sage.combinat
                A subsemigroup of (Ring of integers modulo 15) with 2 generators
                sage: S.list()                                                          # needs sage.combinat
                [3, 5, 9, 0, 10, 12, 6]

            This method is really a shorthand for subsemigroup::

                sage: M2 = R.subsemigroup([R(3), R(5)], one=R.one())                    # needs sage.combinat
                sage: M2 is M                                                           # needs sage.combinat
                True
            """
            return self.subsemigroup(generators, one=self.one())

    class ElementMethods:
        def _div_(left, right):
            r"""
            Default implementation of division, multiplying (on the right) by the inverse.

            INPUT:

            - ``left``, ``right`` -- two elements of the same unital monoid

            .. SEEALSO:: :meth:`__div__`

            EXAMPLES::

                sage: # needs sage.groups
                sage: G = FreeGroup(2)
                sage: x0, x1 = G.group_generators()
                sage: c1 = cartesian_product([x0, x1])
                sage: c2 = cartesian_product([x1, x0])
                sage: c1._div_(c2)
                (x0*x1^-1, x1*x0^-1)

            With this default implementation, division will fail as
            soon as ``right`` is not invertible, even if ``right``
            actually divides ``left``::

                sage: x = cartesian_product([2, 0])
                sage: y = cartesian_product([1, 1])
                sage: x / y
                (2, 0)
                sage: y / x
                Traceback (most recent call last):
                ...
                ZeroDivisionError: rational division by zero

            TESTS::

                sage: c1._div_.__module__                                               # needs sage.groups
                'sage.categories.monoids'
            """
            return left * ~right

        def is_one(self):
            r"""
            Return whether ``self`` is the one of the monoid.

            The default implementation is to compare with ``self.one()``.

            TESTS::

                sage: S = Monoids().example()
                sage: S.one().is_one()
                True
                sage: S("aa").is_one()
                False
            """
            return self == self.parent().one()

        def _pow_int(self, n):
            r"""
            Return ``self`` to the `n^{th}` power.

            INPUT:

            - ``n`` -- a nonnegative integer

            EXAMPLES::

                sage: S = Monoids().example()
                sage: S("a") ^ 5
                'aaaaa'
            """
            return generic_power(self, n)

        def _pow_naive(self, n):
            r"""
            Return ``self`` to the `n^{th}` power (naive implementation).

            INPUT:

            - ``n`` -- a nonnegative integer

            This naive implementation does not use binary
            exponentiation; there are cases where this is actually
            faster due to size explosion.

            EXAMPLES::

                sage: S = Monoids().example()
                sage: x = S("aa")
                sage: [x._pow_naive(i) for i in range(6)]
                ['', 'aa', 'aaaa', 'aaaaaa', 'aaaaaaaa', 'aaaaaaaaaa']
            """
            if not n:
                return self.parent().one()
            result = self
            for i in range(n - 1):
                result *= self
            return result

        def powers(self, n):
            r"""
            Return the list `[x^0, x^1, \ldots, x^{n-1}]`.

            EXAMPLES::

                sage: A = Matrix([[1, 1], [-1, 0]])                                     # needs sage.modules
                sage: A.powers(6)                                                       # needs sage.modules
                [
                [1 0]  [ 1  1]  [ 0  1]  [-1  0]  [-1 -1]  [ 0 -1]
                [0 1], [-1  0], [-1 -1], [ 0 -1], [ 1  0], [ 1  1]
                ]
            """
            if n < 0:
                raise ValueError("negative number of powers requested")
            elif n == 0:
                return []
            x = self.parent().one()
            l = [x]
            for i in range(n - 1):
                x = x * self
                l.append(x)
            return l

        def __invert__(self):
            r"""
            Return the multiplicative inverse of ``self``.

            There is no default implementation, to avoid conflict
            with the default implementation of ``_div_``.

            EXAMPLES::

                sage: A = Matrix([[1, 0], [1, 1]])                                      # needs sage.modules
                sage: ~A                                                                # needs sage.modules
                [ 1 0]
                [-1 1]
            """
            raise NotImplementedError("please implement __invert__")

        def inverse(self):
            """
            Return the multiplicative inverse of ``self``.

            This is an alias for inversion, which can also be invoked
            by ``~x`` for an element ``x``.

            Nota Bene: Element classes should implement ``__invert__`` only.

            EXAMPLES::

                sage: AA(sqrt(~2)).inverse()                                            # needs sage.rings.number_field sage.symbolic
                1.414213562373095?
            """
            # Nota Bene: Element classes should implement ``__invert__`` only.
            return self.__invert__()

    class Commutative(CategoryWithAxiom):
        r"""
        Category of commutative (abelian) monoids.

        A monoid `M` is *commutative* if `xy = yx` for all `x,y \in M`.
        """
        @staticmethod
        def free(index_set=None, names=None, **kwds):
            r"""
            Return a free abelian monoid on `n` generators or with
            the generators indexed by a set `I`.

            A free monoid is constructed by specifying either:

            - the number of generators and/or the names of the generators, or
            - the indexing set for the generators.

            INPUT:

            - ``index_set`` -- (optional) an index set for the generators; if
              an integer, then this represents `\{0, 1, \ldots, n-1\}`

            - ``names`` -- a string or list/tuple/iterable of strings
              (default: ``'x'``); the generator names or name prefix

            EXAMPLES::

                sage: Monoids.Commutative.free(index_set=ZZ)                            # needs sage.combinat
                Free abelian monoid indexed by Integer Ring
                sage: Monoids().Commutative().free(ZZ)                                  # needs sage.combinat
                Free abelian monoid indexed by Integer Ring
                sage: F.<x,y,z> = Monoids().Commutative().free(); F                     # needs sage.combinat
                Free abelian monoid indexed by {'x', 'y', 'z'}
            """
            if names is not None:
                if isinstance(names, str):
                    from sage.rings.integer_ring import ZZ
                    if ',' not in names and index_set in ZZ:
                        names = [names + repr(i) for i in range(index_set)]
                    else:
                        names = names.split(',')
                names = tuple(names)
                if index_set is None:
                    index_set = names

            from sage.monoids.indexed_free_monoid import IndexedFreeAbelianMonoid
            return IndexedFreeAbelianMonoid(index_set, names=names, **kwds)

    class WithRealizations(WithRealizationsCategory):

        class ParentMethods:

            def one(self):
                r"""
                Return the unit of this monoid.

                This default implementation returns the unit of the
                realization of ``self`` given by
                :meth:`~Sets.WithRealizations.ParentMethods.a_realization`.

                EXAMPLES::

                    sage: A = Sets().WithRealizations().example(); A                    # needs sage.modules
                    The subset algebra of {1, 2, 3} over Rational Field
                    sage: A.one.__module__                                              # needs sage.modules
                    'sage.categories.monoids'
                    sage: A.one()                                                       # needs sage.modules
                    F[{}]

                TESTS::

                    sage: A.one() is A.a_realization().one()                            # needs sage.modules
                    True
                    sage: A._test_one()                                                 # needs sage.modules
                """
                return self.a_realization().one()

    class Subquotients(SubquotientsCategory):

        class ParentMethods:

            def one(self):
                """
                Returns the multiplicative unit of this monoid,
                obtained by retracting that of the ambient monoid.

                EXAMPLES::

                    sage: S = Monoids().Subquotients().example() # todo: not implemented
                    sage: S.one()                                # todo: not implemented
                """
                return self.retract(self.ambient().one())

    class Algebras(AlgebrasCategory):

        def extra_super_categories(self):
            """
            The algebra of a monoid is a bialgebra and a monoid.

            EXAMPLES::

                sage: C = Monoids().Algebras(QQ)
                sage: C.extra_super_categories()
                [Category of bialgebras over Rational Field,
                 Category of monoids]
                sage: Monoids().Algebras(QQ).super_categories()
                [Category of bialgebras with basis over Rational Field,
                 Category of semigroup algebras over Rational Field,
                 Category of unital magma algebras over Rational Field]
            """
            from sage.categories.bialgebras import Bialgebras
            return [Bialgebras(self.base_ring()), Monoids()]

        class ParentMethods:

            @cached_method
            def one_basis(self):
                """
                Return the unit of the monoid, which indexes the unit of
                this algebra, as per
                :meth:`AlgebrasWithBasis.ParentMethods.one_basis()
                <sage.categories.algebras_with_basis.AlgebrasWithBasis.ParentMethods.one_basis>`.

                EXAMPLES::

                    sage: # needs sage.modules
                    sage: A = Monoids().example().algebra(ZZ)
                    sage: A.one_basis()
                    ''
                    sage: A.one()
                    B['']
                    sage: A(3)
                    3*B['']
                """
                return self.basis().keys().one()

            @cached_method
            def algebra_generators(self):
                r"""
                Return generators for this algebra.

                For a monoid algebra, the algebra generators are built
                from the monoid generators if available and from the
                semigroup generators otherwise.

                .. SEEALSO::

                    - :meth:`Semigroups.Algebras.ParentMethods.algebra_generators`
                    - :meth:`MagmaticAlgebras.ParentMethods.algebra_generators()
                      <.magmatic_algebras.MagmaticAlgebras.ParentMethods.algebra_generators>`.

                EXAMPLES::

                    sage: M = Monoids().example(); M
                    An example of a monoid:
                    the free monoid generated by ('a', 'b', 'c', 'd')
                    sage: M.monoid_generators()
                    Finite family {'a': 'a', 'b': 'b', 'c': 'c', 'd': 'd'}
                    sage: M.algebra(ZZ).algebra_generators()                            # needs sage.modules
                    Finite family {'a': B['a'], 'b': B['b'], 'c': B['c'], 'd': B['d']}

                    sage: Z12 = Monoids().Finite().example(); Z12
                    An example of a finite multiplicative monoid:
                    the integers modulo 12
                    sage: Z12.monoid_generators()
                    Traceback (most recent call last):
                    ...
                    AttributeError: 'IntegerModMonoid_with_category' object
                    has no attribute 'monoid_generators'...
                    sage: Z12.semigroup_generators()
                    Family (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11)
                    sage: Z12.algebra(QQ).algebra_generators()                          # needs sage.modules
                    Family (B[0], B[1], B[2], B[3], B[4], B[5], B[6], B[7], B[8], B[9], B[10], B[11])


                    sage: A10 = AlternatingGroup(10)                                    # needs sage.groups
                    sage: GroupAlgebras(QQ).example(A10).algebra_generators()           # needs sage.groups sage.modules
                    Family ((8,9,10), (1,2,3,4,5,6,7,8,9))

                    sage: A = DihedralGroup(3).algebra(QQ); A                           # needs sage.groups sage.modules
                    Algebra of Dihedral group of order 6 as a permutation group
                     over Rational Field
                    sage: A.algebra_generators()                                        # needs sage.groups sage.modules
                    Family ((1,2,3), (1,3))
                """
                monoid = self.basis().keys()
                try:
                    generators = monoid.monoid_generators()
                except AttributeError:
                    generators = monoid.semigroup_generators()
                return generators.map(self.monomial)

        class ElementMethods:

            def is_central(self):
                r"""
                Return whether the element ``self`` is central.

                EXAMPLES::

                    sage: SG4 = SymmetricGroupAlgebra(ZZ,4)                             # needs sage.groups sage.modules
                    sage: SG4(1).is_central()                                           # needs sage.groups sage.modules
                    True
                    sage: SG4(Permutation([1,3,2,4])).is_central()                      # needs sage.groups sage.modules
                    False

                    sage: A = GroupAlgebras(QQ).example(); A                            # needs sage.groups sage.modules
                    Algebra of Dihedral group of order 8
                     as a permutation group over Rational Field
                    sage: sum(A.basis()).is_central()                                   # needs sage.groups sage.modules
                    True
                """
                return all(i * self == self * i
                           for i in self.parent().algebra_generators())

    class CartesianProducts(CartesianProductsCategory):
        """
        The category of monoids constructed as Cartesian products of monoids.

        This construction gives the direct product of monoids. See
        :wikipedia:`Direct_product` for more information.
        """
        def extra_super_categories(self):
            """
            A Cartesian product of monoids is endowed with a natural
            group structure.

            EXAMPLES::

                sage: C = Monoids().CartesianProducts()
                sage: C.extra_super_categories()
                [Category of monoids]
                sage: sorted(C.super_categories(), key=str)
                [Category of Cartesian products of semigroups,
                 Category of Cartesian products of unital magmas,
                 Category of monoids]
            """
            return [self.base_category()]

        class ParentMethods:
            @cached_method
            def monoid_generators(self):
                """
                Return the generators of ``self``.

                EXAMPLES::

                    sage: # needs sage.groups
                    sage: M = Monoids.free([1, 2, 3])
                    sage: N = Monoids.free(['a', 'b'])
                    sage: C = cartesian_product([M, N])
                    sage: C.monoid_generators()
                    Family ((F[1], 1), (F[2], 1), (F[3], 1),
                            (1, F['a']), (1, F['b']))

                An example with an infinitely generated group (a better output
                is needed)::

                    sage: N = Monoids.free(ZZ)                                          # needs sage.combinat
                    sage: C = cartesian_product([M, N])                                 # needs sage.combinat sage.groups
                    sage: C.monoid_generators()                                         # needs sage.combinat sage.groups
                    Lazy family (gen(i))_{i in The Cartesian product of (...)}
                """
                F = self.cartesian_factors()
                ids = tuple(M.one() for M in F)

                def lift(i, gen):
                    cur = list(ids)
                    cur[i] = gen
                    return self._cartesian_product_of_elements(cur)
                from sage.sets.family import Family

                # Finitely generated
                cat = FiniteEnumeratedSets()
                if all(M.monoid_generators() in cat or
                       isinstance(M.monoid_generators(), (tuple, list))
                       for M in F):
                    ret = [lift(i, gen) for i, M in enumerate(F)
                           for gen in M.monoid_generators()]
                    return Family(ret)

                # Infinitely generated
                # This does not return a good output, but it is "correct"
                # TODO: Figure out a better way to do things
                from sage.categories.cartesian_product import cartesian_product
                gens_prod = cartesian_product([Family(M.monoid_generators(),
                                                      lambda g: (i, g))
                                               for i, M in enumerate(F)])
                return Family(gens_prod, lift, name="gen")

        class ElementMethods:
            def multiplicative_order(self):
                r"""
                Return the multiplicative order of this element.

                EXAMPLES::

                    sage: # needs sage.groups sage.modules
                    sage: G1 = SymmetricGroup(3)
                    sage: G2 = SL(2, 3)
                    sage: G = cartesian_product([G1, G2])
                    sage: G((G1.gen(0), G2.gen(1))).multiplicative_order()
                    12
                """
                from sage.rings.infinity import Infinity
                orders = [x.multiplicative_order() for x in self.cartesian_factors()]
                if any(o is Infinity for o in orders):
                    return Infinity
                else:
                    from sage.arith.functions import LCM_list
                    return LCM_list(orders)

            def __invert__(self):
                """
                Return the inverse.

                EXAMPLES::

                    sage: # needs sage.groups sage.modules
                    sage: a1 = Permutation((4,2,1,3))
                    sage: a2 = SL(2, 3)([2,1,1,1])
                    sage: h = cartesian_product([a1, a2])
                    sage: ~h
                    ([2, 4, 1, 3], [1 2]
                    [2 2])
                """
                build = self.parent()._cartesian_product_of_elements
                return build([x.__invert__() for x in self.cartesian_factors()])



r"""
Example of facade set
"""
#*****************************************************************************
#  Copyright (C) 2010 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.categories.sets_cat import Sets
from sage.categories.monoids import Monoids
from sage.structure.parent import Parent
from sage.structure.unique_representation import UniqueRepresentation
from sage.rings.integer_ring import ZZ
from sage.rings.infinity import infinity
from sage.sets.finite_enumerated_set import FiniteEnumeratedSet

class PositiveIntegerMonoid(UniqueRepresentation, Parent):
    r"""

    An example of a facade parent: the positive integers viewed as a
    multiplicative monoid

    This class illustrates a minimal implementation of a facade parent
    which models a subset of a set.

    EXAMPLES::

        sage: S = Sets().Facade().example(); S
        An example of facade set: the monoid of positive integers

    TESTS::

        sage: TestSuite(S).run(verbose = True)
        running ._test_an_element() . . . pass
        running ._test_associativity() . . . pass
        running ._test_cardinality() . . . pass
        running ._test_category() . . . pass
        running ._test_construction() . . . pass
        running ._test_elements() . . .
          Running the test suite of self.an_element()
          running ._test_category() . . . pass
          running ._test_eq() . . . pass
          running ._test_new() . . . pass
          running ._test_nonzero_equal() . . . pass
          running ._test_not_implemented_methods() . . . pass
          running ._test_pickling() . . . pass
          pass
        running ._test_elements_eq_reflexive() . . . pass
        running ._test_elements_eq_symmetric() . . . pass
        running ._test_elements_eq_transitive() . . . pass
        running ._test_elements_neq() . . . pass
        running ._test_eq() . . . pass
        running ._test_new() . . . pass
        running ._test_not_implemented_methods() . . . pass
        running ._test_one() . . . pass
        running ._test_pickling() . . . pass
        running ._test_prod() . . . pass
        running ._test_some_elements() . . . pass
    """
    def __init__(self):
        r"""
        EXAMPLES::

            sage: from sage.categories.examples.facade_sets import PositiveIntegerMonoid
            sage: S = PositiveIntegerMonoid(); S
            An example of facade set: the monoid of positive integers

        TESTS::

            sage: TestSuite(S).run()
        """
        Parent.__init__(self, facade=ZZ, category=Monoids())

    def _repr_(self):
        r"""

        EXAMPLES::

            sage: S = Sets().Facade().example()   # indirect doctest

        """
        return "An example of facade set: the monoid of positive integers"

    def _element_constructor_(self, object):
        r"""
        Construction of elements

        Since ``self`` is a strict subset of the parent it is a facade
        for, it is mandatory to override this method. This method
        indirectly influences membership testing (see
        :meth:`Parent.__contains__`).

        EXAMPLES::

            sage: S = Sets().Facade().example(); S
            An example of facade set: the monoid of positive integers
            sage: S(1)
            1
            sage: S(int(1))
            1
            sage: S(2/1)
            2
            sage: (parent(S(1)) == ZZ, parent(S(int(1))) == ZZ, parent(S(2/1)))
            (True, True, Integer Ring)
            sage: S(1), S(int(1)), S(2/1)
            (1, 1, 2)
            sage: 1 in S
            True
            sage: 2/1 in S, int(1) in S
            (True, True)
            sage: -1 in S, 1/2 in S
            (False, False)
        """
        object = ZZ(object)
        if object > ZZ(0):
            return object
        else:
            raise ValueError("%s should be positive")

class IntegersCompletion(UniqueRepresentation, Parent):
    r"""
    An example of a facade parent: the set of integers completed with
    `+-\infty`

    This class illustrates a minimal implementation of a facade parent
    that models the union of several other parents.

    EXAMPLES::

        sage: S = Sets().Facade().example("union"); S
        An example of a facade set: the integers completed by +-infinity

    TESTS::

        sage: TestSuite(S).run(verbose = True)
        running ._test_an_element() . . . pass
        running ._test_cardinality() . . . pass
        running ._test_category() . . . pass
        running ._test_construction() . . . pass
        running ._test_elements() . . .
          Running the test suite of self.an_element()
          running ._test_category() . . . pass
          running ._test_eq() . . . pass
          running ._test_new() . . . pass
          running ._test_nonzero_equal() . . . pass
          running ._test_not_implemented_methods() . . . pass
          running ._test_pickling() . . . pass
          pass
        running ._test_elements_eq_reflexive() . . . pass
        running ._test_elements_eq_symmetric() . . . pass
        running ._test_elements_eq_transitive() . . . pass
        running ._test_elements_neq() . . . pass
        running ._test_eq() . . . pass
        running ._test_new() . . . pass
        running ._test_not_implemented_methods() . . . pass
        running ._test_pickling() . . . pass
        running ._test_some_elements() . . . pass
    """
    def __init__(self):
        r"""
        EXAMPLES::

            sage: from sage.categories.examples.facade_sets import IntegersCompletion
            sage: S = IntegersCompletion(); S
            An example of a facade set: the integers completed by +-infinity

        TESTS::

            sage: TestSuite(S).run()
        """
        # We can't use InfinityRing, because this ring contains 3
        # elements besides +-infinity. We can not use Set either for the
        # moment, because Set([1,2])(1) raises an error
        Parent.__init__(self, facade=(ZZ, FiniteEnumeratedSet([-infinity, +infinity])), category=Sets())

    def _repr_(self):
        r"""

        EXAMPLES::

            sage: S = Sets().Facade().example()   # indirect doctest
        """
        return "An example of a facade set: the integers completed by +-infinity"



"""
Examples of manifolds
"""
# ****************************************************************************
#  Copyright (C) 2015 Travis Scrimshaw <tscrim at ucdavis.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************
from sage.structure.parent import Parent
from sage.structure.unique_representation import UniqueRepresentation
from sage.structure.element_wrapper import ElementWrapper
from sage.categories.manifolds import Manifolds


class Plane(UniqueRepresentation, Parent):
    r"""
    An example of a manifold: the `n`-dimensional plane.

    This class illustrates a minimal implementation of a manifold.

    EXAMPLES::

        sage: from sage.categories.manifolds import Manifolds
        sage: M = Manifolds(QQ).example(); M
        An example of a Rational Field manifold: the 3-dimensional plane

        sage: M.category()
        Category of manifolds over Rational Field

    We conclude by running systematic tests on this manifold::

        sage: TestSuite(M).run()
    """

    def __init__(self, n=3, base_ring=None):
        r"""
        EXAMPLES::

            sage: from sage.categories.manifolds import Manifolds
            sage: M = Manifolds(QQ).example(6); M
            An example of a Rational Field manifold: the 6-dimensional plane

        TESTS::

            sage: TestSuite(M).run()
        """
        self._n = n
        Parent.__init__(self, base=base_ring, category=Manifolds(base_ring))

    def _repr_(self):
        r"""
        TESTS::

            sage: from sage.categories.manifolds import Manifolds
            sage: Manifolds(QQ).example()
            An example of a Rational Field manifold: the 3-dimensional plane
        """
        return "An example of a {} manifold: the {}-dimensional plane".format(
                self.base_ring(), self._n)

    def dimension(self):
        """
        Return the dimension of ``self``.

        EXAMPLES::

            sage: from sage.categories.manifolds import Manifolds
            sage: M = Manifolds(QQ).example()
            sage: M.dimension()
            3
        """
        return self._n

    def an_element(self):
        r"""
        Return an element of the manifold, as per
        :meth:`Sets.ParentMethods.an_element`.

        EXAMPLES::

            sage: from sage.categories.manifolds import Manifolds
            sage: M = Manifolds(QQ).example()
            sage: M.an_element()
            (0, 0, 0)
        """
        zero = self.base_ring().zero()
        return self(tuple([zero]*self._n))

    Element = ElementWrapper


Example = Plane



r"""
Sage categories quickref

- ``sage.categories.primer?``                      a primer on Elements, Parents, and Categories
- ``sage.categories.tutorial?``                    a tutorial on Elements, Parents, and Categories
- ``Category?``                                    technical background on categories
- ``Sets()``, ``Semigroups()``, ``Algebras(QQ)``   some categories
- ``SemiGroups().example()??``                     sample implementation of a semigroup
- ``Hom(A, B)``, ``End(A, Algebras())``            homomorphisms sets
- ``tensor``, ``cartesian_product``                functorial constructions

Module layout:

- :mod:`sage.categories.basic`                the basic categories
- :mod:`sage.categories.all`                  all categories
- :mod:`sage.categories.semigroups`           the ``Semigroups()`` category
- :mod:`sage.categories.examples.semigroups`  the example of ``Semigroups()``
- :mod:`sage.categories.homset`               morphisms, ...
- :mod:`sage.categories.map`
- :mod:`sage.categories.morphism`
- :mod:`sage.categories.functors`
- :mod:`sage.categories.cartesian_product`    functorial constructions
- :mod:`sage.categories.tensor`
- :mod:`sage.categories.dual`
"""
# install the docstring of this module to the containing package
from sage.misc.namespace_package import install_doc
install_doc(__package__, __doc__)

from sage.categories import primer

from sage.misc.lazy_import import lazy_import

from sage.categories.all__sagemath_objects import *

from sage.categories.basic import *

from sage.categories.chain_complexes import ChainComplexes, HomologyFunctor

from sage.categories.simplicial_complexes import SimplicialComplexes

from sage.categories.tensor import tensor
from sage.categories.signed_tensor import tensor_signed

from sage.categories.g_sets import GSets
from sage.categories.pointed_sets import PointedSets

from sage.categories.sets_with_grading import SetsWithGrading

from sage.categories.groupoid import Groupoid
from sage.categories.permutation_groups import PermutationGroups

# enumerated sets
from sage.categories.finite_sets import FiniteSets
from sage.categories.enumerated_sets import EnumeratedSets
from sage.categories.finite_enumerated_sets import FiniteEnumeratedSets
from sage.categories.infinite_enumerated_sets import InfiniteEnumeratedSets

# posets
from sage.categories.posets import Posets
from sage.categories.finite_posets import FinitePosets
from sage.categories.lattice_posets import LatticePosets
from sage.categories.finite_lattice_posets import FiniteLatticePosets

# finite groups/...
from sage.categories.finite_semigroups import FiniteSemigroups
from sage.categories.finite_monoids import FiniteMonoids
from sage.categories.finite_groups import FiniteGroups
from sage.categories.finite_permutation_groups import FinitePermutationGroups

# fields
from sage.categories.number_fields import NumberFields
from sage.categories.function_fields import FunctionFields

# modules
from sage.categories.left_modules import LeftModules
from sage.categories.right_modules import RightModules
from sage.categories.bimodules import Bimodules

from sage.categories.modules import Modules
RingModules = Modules
from sage.categories.vector_spaces import VectorSpaces

# (hopf) algebra structures
from sage.categories.algebras import Algebras
from sage.categories.commutative_algebras import CommutativeAlgebras
from sage.categories.coalgebras import Coalgebras
from sage.categories.bialgebras import Bialgebras
from sage.categories.hopf_algebras import HopfAlgebras
from sage.categories.lie_algebras import LieAlgebras

# specific algebras
from sage.categories.monoid_algebras import MonoidAlgebras
from sage.categories.group_algebras import GroupAlgebras
from sage.categories.matrix_algebras import MatrixAlgebras

# ideals
from sage.categories.ring_ideals import RingIdeals
Ideals = RingIdeals
from sage.categories.commutative_ring_ideals import CommutativeRingIdeals
from sage.categories.algebra_modules import AlgebraModules
from sage.categories.algebra_ideals import AlgebraIdeals
from sage.categories.commutative_algebra_ideals import CommutativeAlgebraIdeals

# schemes and varieties
from sage.categories.modular_abelian_varieties import ModularAbelianVarieties
from sage.categories.schemes import Schemes

# * with basis
from sage.categories.modules_with_basis import ModulesWithBasis
FreeModules = ModulesWithBasis
from sage.categories.hecke_modules            import HeckeModules
from sage.categories.algebras_with_basis      import AlgebrasWithBasis
from sage.categories.coalgebras_with_basis    import CoalgebrasWithBasis
from sage.categories.bialgebras_with_basis    import BialgebrasWithBasis
from sage.categories.hopf_algebras_with_basis import HopfAlgebrasWithBasis

# finite dimensional * with basis
from sage.categories.finite_dimensional_modules_with_basis       import FiniteDimensionalModulesWithBasis
from sage.categories.finite_dimensional_algebras_with_basis      import FiniteDimensionalAlgebrasWithBasis
from sage.categories.finite_dimensional_coalgebras_with_basis    import FiniteDimensionalCoalgebrasWithBasis
from sage.categories.finite_dimensional_bialgebras_with_basis    import FiniteDimensionalBialgebrasWithBasis
from sage.categories.finite_dimensional_hopf_algebras_with_basis import FiniteDimensionalHopfAlgebrasWithBasis

# graded *
from sage.categories.graded_modules       import GradedModules
from sage.categories.graded_algebras      import GradedAlgebras
from sage.categories.graded_coalgebras    import GradedCoalgebras
from sage.categories.graded_bialgebras    import GradedBialgebras
from sage.categories.graded_hopf_algebras import GradedHopfAlgebras

# graded * with basis
from sage.categories.graded_modules_with_basis       import GradedModulesWithBasis
from sage.categories.graded_algebras_with_basis      import GradedAlgebrasWithBasis
from sage.categories.graded_coalgebras_with_basis    import GradedCoalgebrasWithBasis
from sage.categories.graded_bialgebras_with_basis    import GradedBialgebrasWithBasis
from sage.categories.graded_hopf_algebras_with_basis import GradedHopfAlgebrasWithBasis

# Coxeter groups
from sage.categories.coxeter_groups import CoxeterGroups
lazy_import('sage.categories.finite_coxeter_groups', 'FiniteCoxeterGroups')
from sage.categories.weyl_groups import WeylGroups
from sage.categories.finite_weyl_groups import FiniteWeylGroups
from sage.categories.affine_weyl_groups import AffineWeylGroups

# crystal bases
from sage.categories.crystals import Crystals
from sage.categories.highest_weight_crystals import HighestWeightCrystals
from sage.categories.regular_crystals import RegularCrystals
from sage.categories.finite_crystals import FiniteCrystals
from sage.categories.classical_crystals import ClassicalCrystals

# polyhedra
lazy_import('sage.categories.polyhedra', 'PolyhedralSets')

# lie conformal algebras
lazy_import('sage.categories.lie_conformal_algebras', 'LieConformalAlgebras')



r"""
Example of a set with grading
"""
from sage.structure.parent import Parent
from sage.structure.unique_representation import UniqueRepresentation
from sage.categories.sets_with_grading import SetsWithGrading

from sage.rings.integer_ring import IntegerRing
from sage.sets.finite_enumerated_set import FiniteEnumeratedSet


class NonNegativeIntegers(UniqueRepresentation, Parent):
    r"""
    Non negative integers graded by themselves.

    EXAMPLES::

        sage: E = SetsWithGrading().example(); E
        Non negative integers
        sage: E in Sets().Infinite()
        True
        sage: E.graded_component(0)
        {0}
        sage: E.graded_component(100)
        {100}
    """
    def __init__(self):
        r"""
        TESTS::

            sage: TestSuite(SetsWithGrading().example()).run()
        """
        Parent.__init__(self, category=SetsWithGrading().Infinite(),
                        facade=IntegerRing())

    def an_element(self):
        r"""
        Return 0.

        EXAMPLES::

            sage: SetsWithGrading().example().an_element()
            0
        """
        return 0

    def _repr_(self):
        r"""
        TESTS::

            sage: SetsWithGrading().example() # indirect example
            Non negative integers
        """
        return "Non negative integers"

    def graded_component(self, grade):
        r"""
        Return the component with grade ``grade``.

        EXAMPLES::

            sage: N = SetsWithGrading().example()
            sage: N.graded_component(65)
            {65}
        """
        return FiniteEnumeratedSet([grade])

    def grading(self, elt):
        r"""
        Return the grade of ``elt``.

        EXAMPLES::

            sage: N = SetsWithGrading().example()
            sage: N.grading(10)
            10
        """
        return elt

    def generating_series(self, var='z'):
        r"""
        Return `1 / (1-z)`.

        EXAMPLES::

            sage: N = SetsWithGrading().example(); N
            Non negative integers
            sage: f = N.generating_series(); f
            1/(-z + 1)
            sage: LaurentSeriesRing(ZZ,'z')(f)
            1 + z + z^2 + z^3 + z^4 + z^5 + z^6 + z^7 + z^8 + z^9 + z^10 + z^11 + z^12 + z^13 + z^14 + z^15 + z^16 + z^17 + z^18 + z^19 + O(z^20)
        """
        from sage.rings.polynomial.polynomial_ring_constructor import PolynomialRing
        from sage.rings.integer import Integer
        R = PolynomialRing(IntegerRing(), var)
        z = R.gen()
        return Integer(1) / (Integer(1) - z)


Example = NonNegativeIntegers



"""
Simplicial Sets
"""
# ****************************************************************************
#  Copyright (C) 2015 John H. Palmieri <palmieri at math.washington.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from sage.functions.generalized import sign
from sage.matrix.special import identity_matrix
from sage.misc.cachefunc import cached_method
from sage.misc.lazy_import import lazy_import
from sage.categories.category_singleton import Category_singleton
from sage.categories.category_with_axiom import CategoryWithAxiom
from sage.categories.sets_cat import Sets
from sage.categories.homsets import HomsetsCategory
from sage.matrix.constructor import matrix
from sage.misc.misc_c import prod
from sage.rings.infinity import Infinity
from sage.rings.integer import Integer
from sage.rings.integer_ring import ZZ

lazy_import('sage.matrix.constructor', 'matrix')


class SimplicialSets(Category_singleton):
    r"""
    The category of simplicial sets.

    A simplicial set `X` is a collection of sets `X_i`, indexed by
    the non-negative integers, together with maps

    .. math::

        d_i: X_n \to X_{n-1}, \quad 0 \leq i \leq n \quad \text{(face maps)} \\
        s_j: X_n \to X_{n+1}, \quad 0 \leq j \leq n \quad \text{(degeneracy maps)}

    satisfying the *simplicial identities*:

    .. math::

        d_i d_j &= d_{j-1} d_i \quad \text{if } i<j \\
        d_i s_j &= s_{j-1} d_i \quad \text{if } i<j \\
        d_j s_j &= 1 = d_{j+1} s_j \\
        d_i s_j &= s_{j} d_{i-1} \quad \text{if } i>j+1 \\
        s_i s_j &= s_{j+1} s_{i} \quad \text{if } i \leq j

    Morphisms are sequences of maps `f_i : X_i \to Y_i` which commute
    with the face and degeneracy maps.

    EXAMPLES::

        sage: from sage.categories.simplicial_sets import SimplicialSets
        sage: C = SimplicialSets(); C
        Category of simplicial sets

    TESTS::

        sage: TestSuite(C).run()
    """
    @cached_method
    def super_categories(self):
        """
        EXAMPLES::

            sage: from sage.categories.simplicial_sets import SimplicialSets
            sage: SimplicialSets().super_categories()
            [Category of sets]
        """
        return [Sets()]

    class ParentMethods:
        def is_finite(self):
            """
            Return ``True`` if this simplicial set is finite, i.e., has a
            finite number of nondegenerate simplices.

            EXAMPLES::

                sage: simplicial_sets.Torus().is_finite()                               # needs sage.graphs
                True
                sage: C5 = groups.misc.MultiplicativeAbelian([5])                       # needs sage.graphs sage.groups
                sage: simplicial_sets.ClassifyingSpace(C5).is_finite()                  # needs sage.graphs sage.groups
                False
            """
            return SimplicialSets.Finite() in self.categories()

        def is_pointed(self):
            """
            Return ``True`` if this simplicial set is pointed, i.e., has a
            base point.

            EXAMPLES::

                sage: # needs sage.graphs
                sage: from sage.topology.simplicial_set import AbstractSimplex, SimplicialSet
                sage: v = AbstractSimplex(0)
                sage: w = AbstractSimplex(0)
                sage: e = AbstractSimplex(1)
                sage: X = SimplicialSet({e: (v, w)})
                sage: Y = SimplicialSet({e: (v, w)}, base_point=w)
                sage: X.is_pointed()
                False
                sage: Y.is_pointed()
                True
            """
            return SimplicialSets.Pointed() in self.categories()

        def set_base_point(self, point):
            """
            Return a copy of this simplicial set in which the base point is
            set to ``point``.

            INPUT:

            - ``point`` -- a 0-simplex in this simplicial set

            EXAMPLES::

                sage: # needs sage.graphs
                sage: from sage.topology.simplicial_set import AbstractSimplex, SimplicialSet
                sage: v = AbstractSimplex(0, name='v_0')
                sage: w = AbstractSimplex(0, name='w_0')
                sage: e = AbstractSimplex(1)
                sage: X = SimplicialSet({e: (v, w)})
                sage: Y = SimplicialSet({e: (v, w)}, base_point=w)
                sage: Y.base_point()
                w_0
                sage: X_star = X.set_base_point(w)
                sage: X_star.base_point()
                w_0
                sage: Y_star = Y.set_base_point(v)
                sage: Y_star.base_point()
                v_0

            TESTS::

                sage: X.set_base_point(e)                                               # needs sage.graphs
                Traceback (most recent call last):
                ...
                ValueError: the "point" is not a zero-simplex
                sage: pt = AbstractSimplex(0)                                           # needs sage.graphs
                sage: X.set_base_point(pt)                                              # needs sage.graphs
                Traceback (most recent call last):
                ...
                ValueError: the point is not a simplex in this simplicial set
            """
            from sage.topology.simplicial_set import SimplicialSet
            if point.dimension() != 0:
                raise ValueError('the "point" is not a zero-simplex')
            if point not in self._simplices:
                raise ValueError('the point is not a simplex in this '
                                 'simplicial set')
            return SimplicialSet(self.face_data(), base_point=point)

    class Homsets(HomsetsCategory):
        class Endset(CategoryWithAxiom):
            class ParentMethods:
                def one(self):
                    r"""
                    Return the identity morphism in `\operatorname{Hom}(S, S)`.

                    EXAMPLES::

                        sage: T = simplicial_sets.Torus()                               # needs sage.graphs
                        sage: Hom(T, T).identity()                                      # needs sage.graphs
                        Simplicial set endomorphism of Torus
                          Defn: Identity map
                    """
                    from sage.topology.simplicial_set_morphism import SimplicialSetMorphism
                    return SimplicialSetMorphism(domain=self.domain(),
                                                 codomain=self.codomain(),
                                                 identity=True)

    class Finite(CategoryWithAxiom):
        """
        Category of finite simplicial sets.

        The objects are simplicial sets with finitely many
        non-degenerate simplices.
        """
        pass

    class SubcategoryMethods:
        def Pointed(self):
            """
            A simplicial set is *pointed* if it has a distinguished base
            point.

            EXAMPLES::

                sage: from sage.categories.simplicial_sets import SimplicialSets
                sage: SimplicialSets().Pointed().Finite()
                Category of finite pointed simplicial sets
                sage: SimplicialSets().Finite().Pointed()
                Category of finite pointed simplicial sets
            """
            return self._with_axiom("Pointed")

    class Pointed(CategoryWithAxiom):
        class ParentMethods:
            def base_point(self):
                """
                Return this simplicial set's base point

                EXAMPLES::

                    sage: # needs sage.graphs
                    sage: from sage.topology.simplicial_set import AbstractSimplex, SimplicialSet
                    sage: v = AbstractSimplex(0, name='*')
                    sage: e = AbstractSimplex(1)
                    sage: S1 = SimplicialSet({e: (v, v)}, base_point=v)
                    sage: S1.is_pointed()
                    True
                    sage: S1.base_point()
                    *
                """
                return self._basepoint

            def base_point_map(self, domain=None):
                """
                Return a map from a one-point space to this one, with image the
                base point.

                This raises an error if this simplicial set does not have a
                base point.

                INPUT:

                - ``domain`` -- optional, default ``None``. Use
                  this to specify a particular one-point space as
                  the domain. The default behavior is to use the
                  :func:`sage.topology.simplicial_set.Point`
                  function to use a standard one-point space.

                EXAMPLES::

                    sage: # needs sage.graphs
                    sage: T = simplicial_sets.Torus()
                    sage: f = T.base_point_map(); f
                    Simplicial set morphism:
                      From: Point
                      To:   Torus
                      Defn: Constant map at (v_0, v_0)
                    sage: S3 = simplicial_sets.Sphere(3)
                    sage: g = S3.base_point_map()
                    sage: f.domain() == g.domain()
                    True
                    sage: RP3 = simplicial_sets.RealProjectiveSpace(3)                  # needs sage.groups
                    sage: temp = simplicial_sets.Simplex(0)
                    sage: pt = temp.set_base_point(temp.n_cells(0)[0])
                    sage: h = RP3.base_point_map(domain=pt)                             # needs sage.groups
                    sage: f.domain() == h.domain()                                      # needs sage.groups
                    False

                    sage: C5 = groups.misc.MultiplicativeAbelian([5])                   # needs sage.graphs sage.groups
                    sage: BC5 = simplicial_sets.ClassifyingSpace(C5)                    # needs sage.graphs sage.groups
                    sage: BC5.base_point_map()                                          # needs sage.graphs sage.groups
                    Simplicial set morphism:
                      From: Point
                      To:   Classifying space of Multiplicative Abelian group isomorphic to C5
                      Defn: Constant map at 1
                """
                from sage.topology.simplicial_set_examples import Point
                if domain is None:
                    domain = Point()
                else:
                    if len(domain._simplices) > 1:
                        raise ValueError('domain has more than one nondegenerate simplex')
                target = self.base_point()
                return domain.Hom(self).constant_map(point=target)

            def fundamental_group(self, simplify=True):
                r"""
                Return the fundamental group of this pointed simplicial set.

                INPUT:

                - ``simplify`` (bool, optional ``True``) -- if
                  ``False``, then return a presentation of the group
                  in terms of generators and relations. If ``True``,
                  the default, simplify as much as GAP is able to.

                Algorithm: we compute the edge-path group -- see
                Section 19 of [Kan1958]_ and
                :wikipedia:`Fundamental_group`. Choose a spanning tree
                for the connected component of the 1-skeleton
                containing the base point, and then the group's
                generators are given by the non-degenerate
                edges. There are two types of relations: `e=1` if `e`
                is in the spanning tree, and for every 2-simplex, if
                its faces are `e_0`, `e_1`, and `e_2`, then we impose
                the relation `e_0 e_1^{-1} e_2 = 1`, where we first
                set `e_i=1` if `e_i` is degenerate.

                EXAMPLES::

                    sage: S1 = simplicial_sets.Sphere(1)                                # needs sage.graphs
                    sage: eight = S1.wedge(S1)                                          # needs sage.graphs
                    sage: eight.fundamental_group()  # free group on 2 generators       # needs sage.graphs sage.groups
                    Finitely presented group < e0, e1 |  >

                The fundamental group of a disjoint union of course depends on
                the choice of base point::

                    sage: T = simplicial_sets.Torus()                                   # needs sage.graphs
                    sage: K = simplicial_sets.KleinBottle()                             # needs sage.graphs
                    sage: X = T.disjoint_union(K)                                       # needs sage.graphs

                    sage: # needs sage.graphs
                    sage: X_0 = X.set_base_point(X.n_cells(0)[0])
                    sage: X_0.fundamental_group().is_abelian()                          # needs sage.groups
                    True
                    sage: X_1 = X.set_base_point(X.n_cells(0)[1])
                    sage: X_1.fundamental_group().is_abelian()                          # needs sage.groups
                    False

                    sage: RP3 = simplicial_sets.RealProjectiveSpace(3)                  # needs sage.graphs sage.groups
                    sage: RP3.fundamental_group()                                       # needs sage.graphs sage.groups
                    Finitely presented group < e | e^2 >

                Compute the fundamental group of some classifying spaces::

                    sage: C5 = groups.misc.MultiplicativeAbelian([5])                   # needs sage.graphs sage.groups
                    sage: BC5 = C5.nerve()                                              # needs sage.graphs sage.groups
                    sage: BC5.fundamental_group()                                       # needs sage.graphs sage.groups
                    Finitely presented group < e0 | e0^5 >

                    sage: # needs sage.graphs sage.groups
                    sage: Sigma3 = groups.permutation.Symmetric(3)
                    sage: BSigma3 = Sigma3.nerve()
                    sage: pi = BSigma3.fundamental_group(); pi
                    Finitely presented group < e1, e2 | e2^2, e1^3, (e2*e1)^2 >
                    sage: pi.order()
                    6
                    sage: pi.is_abelian()
                    False

                The sphere has a trivial fundamental group::

                    sage: S2 = simplicial_sets.Sphere(2)                                # needs sage.graphs
                    sage: S2.fundamental_group()                                        # needs sage.graphs sage.groups
                    Finitely presented group <  |  >
                """
                # Import this here to prevent importing libgap upon startup.
                from sage.groups.free_group import FreeGroup
                if not self.n_cells(1):
                    return FreeGroup([]).quotient([])
                FG = self._universal_cover_dict()[0]
                if simplify:
                    return FG.simplified()
                else:
                    return FG

            def _universal_cover_dict(self):
                r"""
                Return the fundamental group and dictionary sending each edge to
                the corresponding group element

                TESTS::

                    sage: RP2 = simplicial_sets.RealProjectiveSpace(2)                  # needs sage.groups
                    sage: RP2._universal_cover_dict()                                   # needs sage.groups
                    (Finitely presented group < e | e^2 >, {f: e})
                    sage: RP2.nondegenerate_simplices()                                 # needs sage.groups
                    [1, f, f * f]
                """
                from sage.groups.free_group import FreeGroup
                graph = self.graph()
                if not self.is_connected():
                    graph = graph.subgraph(self.base_point())
                edges = [e[2] for e in graph.edges(sort=False)]
                spanning_tree = [e[2] for e in graph.min_spanning_tree()]
                gens = [e for e in edges if e not in spanning_tree]
                gens_dict = dict(zip(gens, range(len(gens))))
                FG = FreeGroup(len(gens), 'e')
                rels = []

                for f in self.n_cells(2):
                    z = {}
                    for i, sigma in enumerate(self.faces(f)):
                        if sigma in spanning_tree:
                            z[i] = FG.one()
                        elif sigma.is_degenerate():
                            z[i] = FG.one()
                        elif sigma in edges:
                            z[i] = FG.gen(gens_dict[sigma])
                        else:
                            # sigma is not in the correct connected component.
                            z[i] = FG.one()
                    rels.append(z[0]*z[1].inverse()*z[2])
                G = FG.quotient(rels)
                char = {g: G.gen(i) for i, g in enumerate(gens)}
                for e in edges:
                    if e not in gens:
                        char[e] = G.one()
                return (G, char)

            def universal_cover_map(self):
                r"""
                Return the universal covering map of the simplicial set.

                It requires the fundamental group to be finite.

                EXAMPLES::

                    sage: RP2 = simplicial_sets.RealProjectiveSpace(2)                  # needs sage.groups
                    sage: phi = RP2.universal_cover_map(); phi                          # needs sage.groups
                    Simplicial set morphism:
                      From: Simplicial set with 6 non-degenerate simplices
                      To:   RP^2
                      Defn: [(1, 1), (1, e), (f, 1), (f, e), (f * f, 1), (f * f, e)]
                            --> [1, 1, f, f, f * f, f * f]
                    sage: phi.domain().face_data()                                      # needs sage.groups
                        {(1, 1): None,
                         (1, e): None,
                         (f, 1): ((1, e), (1, 1)),
                         (f, e): ((1, 1), (1, e)),
                         (f * f, 1): ((f, e), s_0 (1, 1), (f, 1)),
                         (f * f, e): ((f, 1), s_0 (1, e), (f, e))}

                """
                edges = self.n_cells(1)
                if not edges:
                    return self.identity()
                G, char = self._universal_cover_dict()
                return self.covering_map(char)

            def covering_map(self, character):
                r"""
                Return the covering map associated to a character.

                The character is represented by a dictionary that assigns an
                element of a finite group to each nondegenerate 1-dimensional
                cell. It should correspond to an epimorphism from the fundamental
                group.

                INPUT:

                - ``character`` -- a dictionary


                EXAMPLES::

                    sage: # needs sage.graphs sage.groups
                    sage: S1 = simplicial_sets.Sphere(1)
                    sage: W = S1.wedge(S1)
                    sage: G = CyclicPermutationGroup(3)
                    sage: a, b = W.n_cells(1)
                    sage: C = W.covering_map({a : G.gen(0), b : G.one()}); C
                    Simplicial set morphism:
                      From: Simplicial set with 9 non-degenerate simplices
                      To:   Wedge: (S^1 v S^1)
                      Defn: [(*, ()), (*, (1,2,3)), (*, (1,3,2)), (sigma_1, ()),
                             (sigma_1, ()), (sigma_1, (1,2,3)), (sigma_1, (1,2,3)),
                             (sigma_1, (1,3,2)), (sigma_1, (1,3,2))]
                            --> [*, *, *, sigma_1, sigma_1, sigma_1, sigma_1, sigma_1, sigma_1]
                    sage: C.domain()
                    Simplicial set with 9 non-degenerate simplices
                    sage: C.domain().face_data()
                    {(*, ()): None,
                     (*, (1,2,3)): None,
                     (*, (1,3,2)): None,
                     (sigma_1, ()): ((*, (1,2,3)), (*, ())),
                     (sigma_1, ()): ((*, ()), (*, ())),
                     (sigma_1, (1,2,3)): ((*, (1,3,2)), (*, (1,2,3))),
                     (sigma_1, (1,2,3)): ((*, (1,2,3)), (*, (1,2,3))),
                     (sigma_1, (1,3,2)): ((*, ()), (*, (1,3,2))),
                     (sigma_1, (1,3,2)): ((*, (1,3,2)), (*, (1,3,2)))}
                """
                from sage.topology.simplicial_set import AbstractSimplex, SimplicialSet
                from sage.topology.simplicial_set_morphism import SimplicialSetMorphism
                char = dict(character.items())
                G = list(char.values())[0].parent()
                if not G.is_finite():
                    raise NotImplementedError("can only compute universal covers of spaces with finite fundamental group")
                cells_dict = {}
                faces_dict = {}

                for s in self.n_cells(0):
                    for g in G:
                        cell = AbstractSimplex(0, name="({}, {})".format(s, g))
                        cells_dict[(s, g)] = cell
                        char[s] = G.one()

                for d in range(1, self.dimension() + 1):
                    for s in self.n_cells(d):
                        if s not in char.keys():
                            if d == 1 and s.is_degenerate():
                                char[s] = G.one()
                            elif s.is_degenerate():
                                if 0 in s.degeneracies():
                                    char[s] = G.one()
                                else:
                                    char[s] = char[s.nondegenerate()]
                            else:
                                char[s] = char[self.face(s, d)]
                        if s.is_nondegenerate():
                            for g in G:
                                cell = AbstractSimplex(d, name="({}, {})".format(s, g))
                                cells_dict[(s, g)] = cell
                                fd = []
                                faces = self.faces(s)
                                f0 = faces[0]
                                for h in G:
                                    if h == g*char[s]:
                                        lifted = h
                                        break
                                grelems = [cells_dict[(f0.nondegenerate(), lifted)].apply_degeneracies(*f0.degeneracies())]
                                for f in faces[1:]:
                                    grelems.append(cells_dict[(f.nondegenerate(), g)].apply_degeneracies(*f.degeneracies()))
                                faces_dict[cell] = grelems
                cover = SimplicialSet(faces_dict, base_point=cells_dict[(self.base_point(), G.one())])
                cover_map_data = {c: s[0] for (s, c) in cells_dict.items()}
                return SimplicialSetMorphism(data=cover_map_data, domain=cover, codomain=self)

            def cover(self, character):
                r"""
                Return the cover of the simplicial set associated to a character
                of the fundamental group.

                The character is represented by a dictionary, that assigns an
                element of a finite group to each nondegenerate 1-dimensional
                cell. It should correspond to an epimorphism from the fundamental
                group.

                INPUT:

                - ``character`` -- a dictionary

                EXAMPLES::

                    sage: # needs sage.graphs sage.groups
                    sage: S1 = simplicial_sets.Sphere(1)
                    sage: W = S1.wedge(S1)
                    sage: G = CyclicPermutationGroup(3)
                    sage: (a, b) = W.n_cells(1)
                    sage: C = W.cover({a : G.gen(0), b : G.gen(0)^2})
                    sage: C.face_data()
                    {(*, ()): None,
                     (*, (1,2,3)): None,
                     (*, (1,3,2)): None,
                     (sigma_1, ()): ((*, (1,2,3)), (*, ())),
                     (sigma_1, ()): ((*, (1,3,2)), (*, ())),
                     (sigma_1, (1,2,3)): ((*, (1,3,2)), (*, (1,2,3))),
                     (sigma_1, (1,2,3)): ((*, ()), (*, (1,2,3))),
                     (sigma_1, (1,3,2)): ((*, ()), (*, (1,3,2))),
                     (sigma_1, (1,3,2)): ((*, (1,2,3)), (*, (1,3,2)))}
                    sage: C.homology(1)                                                 # needs sage.modules
                    Z x Z x Z x Z
                    sage: C.fundamental_group()
                    Finitely presented group < e0, e1, e2, e3 |  >
                """
                return self.covering_map(character).domain()

            def universal_cover(self):
                r"""
                Return the universal cover of the simplicial set.
                The fundamental group must be finite in order to ensure that the
                universal cover is a simplicial set of finite type.

                EXAMPLES::

                    sage: # needs sage.groups
                    sage: RP3 = simplicial_sets.RealProjectiveSpace(3)
                    sage: C = RP3.universal_cover(); C
                    Simplicial set with 8 non-degenerate simplices
                    sage: C.face_data()
                    {(1, 1): None,
                     (1, e): None,
                     (f, 1): ((1, e), (1, 1)),
                     (f, e): ((1, 1), (1, e)),
                     (f * f, 1): ((f, e), s_0 (1, 1), (f, 1)),
                     (f * f, e): ((f, 1), s_0 (1, e), (f, e)),
                     (f * f * f, 1): ((f * f, e), s_0 (f, 1), s_1 (f, 1), (f * f, 1)),
                     (f * f * f, e): ((f * f, 1), s_0 (f, e), s_1 (f, e), (f * f, e))}
                    sage: C.fundamental_group()
                    Finitely presented group <  |  >
                """
                return self.universal_cover_map().domain()

            def _canonical_twisting_operator(self):
                r"""
                The canonical twisting operator corresponds to the abelianization of the fundamental
                group. It assigns each edge to the corresponding element in the algebra of the
                abelianization of the fundamental group, which is a quotient of a Laurent polynomial
                ring.

                EXAMPLES::

                    sage: X = simplicial_sets.Torus()
                    sage: d = X._canonical_twisting_operator()
                    sage: d
                    {(s_0 v_0, sigma_1): f3, (sigma_1, s_0 v_0): f2*f3^-1, (sigma_1, sigma_1): f2}
                    sage: list(d.values())[0].parent()
                    Multivariate Laurent Polynomial Ring in f2, f3 over Integer Ring
                    sage: Y = simplicial_sets.RealProjectiveSpace(2)
                    sage: d2 = Y._canonical_twisting_operator()
                    sage: d2
                    {f: F1bar}
                    sage: list(d2.values())[0].parent()
                    Quotient of Univariate Laurent Polynomial Ring in F1 over Integer Ring by the ideal (-1 + F1^2)

                """
                G, d = self._universal_cover_dict()
                phi = G.abelianization_map()
                abelG, R, I, images = G.abelianization_to_algebra(ZZ)
                QRP = R.quotient_ring(I)
                res = {}
                for s, el in d.items():
                    res[s] = QRP(prod(images[abs(a)-1]**sign(a) for a in el.Tietze()))
                return res

            def twisted_chain_complex(self, twisting_operator=None, dimensions=None, augmented=False,
                                      cochain=False, verbose=False, subcomplex=None,
                                      check=False):
                r"""
                Return the normalized chain complex twisted by some operator.

                A twisting operator is a map from the set of simplices to some algebra.
                The differentials are then twisted by this operator.

                INPUT:

                - ``twisting_operator`` -- a dictionary, associating the twist of each
                  simplex. If it is not given, the canonical one (associated to the
                  laurent polynomial ring abelianization of the fundamental group, ignoring
                  torsion) is used.

                - ``dimensions`` -- if ``None``, compute the chain complex in all
                  dimensions.  If a list or tuple of integers, compute the
                  chain complex in those dimensions, setting the chain groups
                  in all other dimensions to zero.

                - ``augmented`` (optional, default ``False``) -- if ``True``,
                  return the augmented chain complex (that is, include a class
                  in dimension `-1` corresponding to the empty cell).

                - ``cochain`` (optional, default ``False``) -- if ``True``,
                  return the cochain complex (that is, the dual of the chain
                  complex).

                - ``verbose`` (optional, default ``False``) -- ignored.

                - ``subcomplex`` (optional, default ``None``) -- if present,
                  compute the chain complex relative to this subcomplex.

                - ``check`` (optional, default ``False``) -- If ``True``, make
                  sure that the chain complex is actually a chain complex:
                  the differentials are composable and their product is zero.

                The normalized chain complex of a simplicial set is isomorphic
                to the chain complex obtained by modding out by degenerate
                simplices, and the latter is what is actually constructed
                here.

                EXAMPLES::

                    sage: W = simplicial_sets.Sphere(1).wedge(simplicial_sets.Sphere(2))
                    sage: W.nondegenerate_simplices()
                    [*, sigma_1, sigma_2]
                    sage: s1 = W.nondegenerate_simplices()[1]
                    sage: L.<t> = LaurentPolynomialRing(QQ)
                    sage: tw = {s1:t}
                    sage: ChC = W.twisted_chain_complex(tw)
                    sage: ChC.differential(1)
                    [-1 + t]
                    sage: ChC.differential(2)
                    [0]

                ::

                    sage: X = simplicial_sets.Torus()
                    sage: C = X.twisted_chain_complex()
                    sage: C.differential(1)
                    [      f3 - 1 f2*f3^-1 - 1       f2 - 1]
                    sage: C.differential(2)
                    [       1 f2*f3^-1]
                    [      f3        1]
                    [      -1       -1]
                    sage: C.differential(3)
                    []

                ::

                    sage: Y = simplicial_sets.RealProjectiveSpace(2)
                    sage: C = Y.twisted_chain_complex()
                    sage: C.differential(1)
                    [-1 + F1]
                    sage: C.differential(2)
                    [1 + F1]
                    sage: C.differential(3)
                    []

                """
                from sage.homology.chain_complex import ChainComplex
                from sage.structure.element import get_coercion_model
                cm = get_coercion_model()

                if twisting_operator:
                    twop = twisting_operator
                else:
                    di = self._canonical_twisting_operator()
                    if hasattr(list(di.values())[0], "lift"):
                        twop = {a: b.lift() for a, b in di.items()}
                    else:
                        twop = di

                def twist(s):
                    if s in twop:
                        return twop[s]
                    if s.dimension() > 1:
                        return twist(self.face(s,s.dimension()))
                    return 1
                base_ring = cm.common_parent(*twop.values())

                if dimensions is None:
                    if not self.cells():  # Empty
                        if cochain:
                            return ChainComplex({-1: matrix(base_ring, 0, 0)},
                                                degree_of_differential=1)
                        return ChainComplex({0: matrix(base_ring, 0, 0)},
                                            degree_of_differential=-1)
                    dimensions = list(range(self.dimension() + 1))
                else:
                    if not isinstance(dimensions, (list, tuple, range)):
                        dimensions = list(range(dimensions - 1, dimensions + 2))
                    else:
                        dimensions = [n for n in dimensions if n >= 0]
                    if not dimensions:
                        # Return the empty chain complex.
                        if cochain:
                            return ChainComplex(base_ring=base_ring, degree=1)
                        else:
                            return ChainComplex(base_ring=base_ring, degree=-1)

                differentials = {}
                # Convert the tuple self._data to a dictionary indexed by the
                # non-degenerate simplices.
                if subcomplex:
                    X = self.quotient(subcomplex)
                    face_data = X.face_data()
                    nondegens = X.nondegenerate_simplices()
                else:
                    face_data = self.face_data()
                    nondegens = self.nondegenerate_simplices()
                # simplices: dictionary indexed by dimension, values the list
                # of non-degenerate simplices in that dimension.
                simplices = {}
                for sigma in nondegens:
                    if sigma.dimension() in simplices:
                        simplices[sigma.dimension()].append(sigma)
                    else:
                        simplices[sigma.dimension()] = [sigma]
                first = dimensions.pop(0)
                if first in simplices:
                    rank = len(simplices[first])
                    current = sorted(simplices[first])
                else:
                    rank = 0
                    current = []
                if augmented and first == 0:
                    differentials[first-1] = matrix(base_ring, 0, 1)
                    differentials[first] = matrix(base_ring, 1, rank,
                                                  [1] * rank)
                else:
                    differentials[first] = matrix(base_ring, 0, rank)

                for d in dimensions:
                    old_rank = rank
                    faces = {_[1]: _[0] for _ in enumerate(current)}
                    if d in simplices:
                        current = sorted(simplices[d])
                        rank = len(current)
                        # old_rank: number of simplices in dimension d-1.
                        # faces: list of simplices in dimension d-1.
                        # rank: number of simplices in dimension d.
                        # current: list of simplices in dimension d.
                        if not faces:
                            differentials[d] = matrix(base_ring, old_rank, rank)
                        else:
                            matrix_data = {}
                            for col, sigma in enumerate(current):
                                sign = 1
                                twists = len(face_data[sigma]) * [1]
                                twists[0] = twist(sigma)
                                for (ch, tau) in zip(twists, face_data[sigma]):
                                    if tau.is_nondegenerate():
                                        row = faces[tau]
                                        if (row, col) in matrix_data:
                                            matrix_data[(row, col)] += sign*ch
                                        else:
                                            matrix_data[(row, col)] = sign*ch
                                    sign *= -1

                            differentials[d] = matrix(base_ring, old_rank,
                                                      rank, matrix_data, sparse=False)

                    else:
                        rank = 0
                        current = []
                        differentials[d] = matrix(base_ring, old_rank, rank, sparse=False)

                if cochain:
                    new_diffs = {}
                    for d in differentials:
                        new_diffs[d-1] = differentials[d].transpose()
                    return ChainComplex(new_diffs, degree_of_differential=1,
                                        check=check)
                return ChainComplex(differentials, degree_of_differential=-1,
                                    check=check)

            def twisted_homology(self, n, reduced=False):
                r"""
                The `n`-th twisted homology module of the simplicial set with respect to
                the abelianization of the fundamental_group.

                It is a module over a polynomial ring, including relations to make some
                variables the multiplicative inverses of others.

                INPUT:

                - ``n`` - a positive integer.

                - ``reduced`` - (default: False) if set to True, the presentation matrix
                  will be reduced.

                EXAMPLES::

                    sage: X = simplicial_sets.Sphere(1).wedge(simplicial_sets.Sphere(2))
                    sage: X.twisted_homology(1)
                    Quotient module by Submodule of Ambient free module of rank 0 over the integral domain Multivariate Polynomial Ring in f1, f1inv over Integer Ring
                    Generated by the rows of the matrix:
                    []
                    sage: X.twisted_homology(2)
                    Quotient module by Submodule of Ambient free module of rank 1 over the integral domain Multivariate Polynomial Ring in f1, f1inv over Integer Ring
                    Generated by the rows of the matrix:
                    [f1*f1inv - 1]

                ::

                    sage: Y = simplicial_sets.Torus()
                    sage: Y.twisted_homology(1)
                    Quotient module by Submodule of Ambient free module of rank 5 over the integral domain Multivariate Polynomial Ring in f2, f2inv, f3, f3inv over Integer Ring
                    Generated by the rows of the matrix:
                    [           1            0            0            0            0]
                    [           0            1            0            0            0]
                    [           0            0            1            0            0]
                    [           0            0            0            1            0]
                    [           0            0            0            0            1]
                    [f2*f2inv - 1            0            0            0            0]
                    [           0 f2*f2inv - 1            0            0            0]
                    [           0            0 f2*f2inv - 1            0            0]
                    [           0            0            0 f2*f2inv - 1            0]
                    [           0            0            0            0 f2*f2inv - 1]
                    [f3*f3inv - 1            0            0            0            0]
                    [           0 f3*f3inv - 1            0            0            0]
                    [           0            0 f3*f3inv - 1            0            0]
                    [           0            0            0 f3*f3inv - 1            0]
                    [           0            0            0            0 f3*f3inv - 1]
                    sage: Y.twisted_homology(2)
                    Quotient module by Submodule of Ambient free module of rank 0 over the integral domain Multivariate Polynomial Ring in f2, f2inv, f3, f3inv over Integer Ring
                    Generated by the rows of the matrix:
                    []
                    sage: Y.twisted_homology(1, reduced=True)
                    Quotient module by Submodule of Ambient free module of rank 5 over the integral domain Multivariate Polynomial Ring in f2, f2inv, f3, f3inv over Integer Ring
                    Generated by the rows of the matrix:
                    [1 0 0 0 0]
                    [0 1 0 0 0]
                    [0 0 1 0 0]
                    [0 0 0 1 0]
                    [0 0 0 0 1]

                TESTS::

                    sage: X = simplicial_sets.PresentationComplex(groups.presentation.FGAbelian((3,2)))
                    sage: TW2 = X.twisted_homology(2, reduced=True)
                    sage: M = TW2.relations_matrix()
                    sage: from sage.libs.singular.function import singular_function
                    sage: vdim = singular_function("vdim")
                    sage: vdim(M.T, ring=M.base_ring())
                    // ** considering the image in Q[...]
                    // ** _ is no standard basis
                    5
                    sage: X.universal_cover().homology(2)
                    Z^5
                    sage: from sage.libs.singular.function import singular_function

                """
                from sage.libs.singular.function import singular_function
                from sage.libs.singular.option import opt_verb
                opt_verb['not_warn_sb'] = True
                singstd = singular_function("std")
                singsyz = singular_function("syz")
                singred = singular_function("reduce")
                singlift = singular_function("lift")
                G, d = self._universal_cover_dict()
                phi = G.abelianization_map()
                abelG, R, I, images = G.abelianization_to_algebra(ZZ)
                CC = self.twisted_chain_complex()
                M1 = CC.differential(n).T
                M2 = CC.differential(n + 1).T

                def convert_to_polynomial(p):
                    if hasattr(p, "lift"):
                        return p.lift()._as_extended_polynomial()
                    else:
                        return p._as_extended_polynomial()
                M1 = M1.apply_map(convert_to_polynomial)
                M2 = M2.apply_map(convert_to_polynomial)
                RP = R._extended_ring
                IP = RP.ideal([convert_to_polynomial(g) for g in I])
                JP = R._extended_ring_ideal
                GB = (IP+JP).groebner_basis()
                GBI = RP.ideal(GB)

                def reduce_laurent(a):
                    return singred(a, GBI, ring=RP)

                def group_to_polynomial(el, RP):
                    res = RP.one()
                    for a in el.Tietze():
                        if a > 0:
                            res *= RP.gen(2*a-2)
                        else:
                            res *= RP.gen(-2*a-1)
                    return res

                def mkernel(M):
                    if M.nrows() == 0:
                        return matrix(M.base_ring(), 0, 0)
                    if M.ncols() == 0:
                        return M.T
                    res = M
                    n = res.ncols()
                    for g in (IP+JP).gens():
                        res = res.stack(g*identity_matrix(n))
                    syz = matrix(singsyz(res.T, ring=res.base_ring())).T
                    trimmed = syz.T.submatrix(0, 0, syz.ncols(), M.nrows())
                    trimmed = trimmed.apply_map(reduce_laurent)
                    to_delete = [i for (i, r) in enumerate(trimmed.rows()) if not r]
                    return trimmed.delete_rows(to_delete)

                def lift_to_submodule(S, M):
                    if S.nrows() == 0 or S.ncols() == 0:
                        return S
                    res = M
                    for g in GB:
                        res = res.stack(g*identity_matrix(M.ncols()))
                    singres = matrix(singlift(res.T, S.T,ring=res.base_ring()))
                    return singres.submatrix(0, 0, M.nrows(), S.nrows())

                def mgb(M):
                    if M.nrows() == 0 or M.ncols() == 0:
                        return M
                    res = M
                    for g in GB:
                        res = res.stack(g*identity_matrix(M.ncols()))
                    sres = matrix(singstd(res.T, ring=RP))
                    to_delete = [i for i, r in enumerate(sres.apply_map(reduce_laurent)) if not r]
                    return sres.delete_rows(to_delete)
                    M2 = border_matrix(n+1)
                if M1.nrows() == 0:
                    opt_verb.reset_default()
                    return (RP**0).quotient_module([])
                K = mkernel(M1)
                DK = mkernel(K)
                if M2.nrows() > 0:
                    S = lift_to_submodule(M2, K)
                    if S.nrows() > 0 and S.ncols() > 0:
                        res = mgb(DK.stack(S.T))
                    else:
                        res = DK
                else:
                    res = mgb(DK)
                resmat = mgb(res.apply_map(reduce_laurent))
                AM = RP ** K.nrows()
                if resmat.ncols() == 0:
                    SM = AM.submodule([])
                    opt_verb.reset_default()
                    return AM.quotient_module(SM)
                for g in (IP+JP).gens():
                    resmat = resmat.stack(g * identity_matrix(resmat.ncols()))
                if reduced:
                    resmat = matrix(singstd(resmat.T, ring=RP))
                SM = AM.submodule(resmat)
                opt_verb.reset_default()
                return AM.quotient_module(SM)

            def is_simply_connected(self):
                """
                Return ``True`` if this pointed simplicial set is simply connected.

                .. WARNING::

                    Determining simple connectivity is not always
                    possible, because it requires determining when a
                    group, as given by generators and relations, is
                    trivial. So this conceivably may give a false
                    negative in some cases.

                EXAMPLES::

                    sage: # needs sage.graphs sage.groups
                    sage: T = simplicial_sets.Torus()
                    sage: T.is_simply_connected()
                    False
                    sage: T.suspension().is_simply_connected()
                    True
                    sage: simplicial_sets.KleinBottle().is_simply_connected()
                    False

                    sage: # needs sage.graphs
                    sage: S2 = simplicial_sets.Sphere(2)
                    sage: S3 = simplicial_sets.Sphere(3)
                    sage: (S2.wedge(S3)).is_simply_connected()                          # needs sage.groups
                    True
                    sage: X = S2.disjoint_union(S3)
                    sage: X = X.set_base_point(X.n_cells(0)[0])
                    sage: X.is_simply_connected()
                    False

                    sage: C3 = groups.misc.MultiplicativeAbelian([3])                   # needs sage.graphs sage.groups
                    sage: BC3 = simplicial_sets.ClassifyingSpace(C3)                    # needs sage.graphs sage.groups
                    sage: BC3.is_simply_connected()                                     # needs sage.graphs sage.groups
                    False
                """
                if not self.is_connected():
                    return False
                try:
                    if not self.is_pointed():
                        space = self.set_base_point(self.n_cells(0)[0])
                    else:
                        space = self
                    return bool(space.fundamental_group().IsTrivial())
                except AttributeError:
                    try:
                        return space.fundamental_group().order() == 1
                    except (NotImplementedError, RuntimeError):
                        # I don't know of any simplicial sets for which the
                        # code reaches this point, but there are certainly
                        # groups for which these errors are raised. 'IsTrivial'
                        # works for all of the examples I've seen, though.
                        raise ValueError('unable to determine if the fundamental '
                                         'group is trivial')

            def connectivity(self, max_dim=None):
                """
                Return the connectivity of this pointed simplicial set.

                INPUT:

                - ``max_dim`` -- specify a maximum dimension through
                  which to check. This is required if this simplicial
                  set is simply connected and not finite.

                The dimension of the first nonzero homotopy group. If
                simply connected, this is the same as the dimension of
                the first nonzero homology group.

                .. WARNING::

                   See the warning for the :meth:`is_simply_connected` method.

                The connectivity of a contractible space is ``+Infinity``.

                EXAMPLES::

                    sage: # needs sage.graphs sage.groups
                    sage: simplicial_sets.Sphere(3).connectivity()
                    2
                    sage: simplicial_sets.Sphere(0).connectivity()
                    -1
                    sage: K = simplicial_sets.Simplex(4)
                    sage: K = K.set_base_point(K.n_cells(0)[0])
                    sage: K.connectivity()
                    +Infinity
                    sage: X = simplicial_sets.Torus().suspension(2)
                    sage: X.connectivity()
                    2

                    sage: C2 = groups.misc.MultiplicativeAbelian([2])                   # needs sage.graphs sage.groups
                    sage: BC2 = simplicial_sets.ClassifyingSpace(C2)                    # needs sage.graphs sage.groups
                    sage: BC2.connectivity()                                            # needs sage.graphs sage.groups
                    0
                """
                if not self.is_connected():
                    return Integer(-1)
                if not self.is_simply_connected():
                    return Integer(0)
                if max_dim is None:
                    if self.is_finite():
                        max_dim = self.dimension()
                    else:
                        # Note: at the moment, this will never be reached,
                        # because our only examples (so far) of infinite
                        # simplicial sets are not simply connected.
                        raise ValueError('this simplicial set may be infinite, '
                                         'so specify a maximum dimension through '
                                         'which to check')

                H = self.homology(range(2, max_dim + 1))
                for i in range(2, max_dim + 1):
                    if i in H and H[i].order() != 1:
                        return i-1
                return Infinity

        class Finite(CategoryWithAxiom):
            class ParentMethods():

                def unset_base_point(self):
                    """
                    Return a copy of this simplicial set in which the base point has
                    been forgotten.

                    EXAMPLES::

                        sage: # needs sage.graphs
                        sage: from sage.topology.simplicial_set import AbstractSimplex, SimplicialSet
                        sage: v = AbstractSimplex(0, name='v_0')
                        sage: w = AbstractSimplex(0, name='w_0')
                        sage: e = AbstractSimplex(1)
                        sage: Y = SimplicialSet({e: (v, w)}, base_point=w)
                        sage: Y.is_pointed()
                        True
                        sage: Y.base_point()
                        w_0
                        sage: Z = Y.unset_base_point()
                        sage: Z.is_pointed()
                        False
                    """
                    from sage.topology.simplicial_set import SimplicialSet
                    return SimplicialSet(self.face_data())

                def fat_wedge(self, n):
                    """
                    Return the `n`-th fat wedge of this pointed simplicial set.

                    This is the subcomplex of the `n`-fold product `X^n`
                    consisting of those points in which at least one
                    factor is the base point. Thus when `n=2`, this is the
                    wedge of the simplicial set with itself, but when `n`
                    is larger, the fat wedge is larger than the `n`-fold
                    wedge.

                    EXAMPLES::

                        sage: # needs sage.graphs
                        sage: S1 = simplicial_sets.Sphere(1)
                        sage: S1.fat_wedge(0)
                        Point
                        sage: S1.fat_wedge(1)
                        S^1
                        sage: S1.fat_wedge(2).fundamental_group()                       # needs sage.groups
                        Finitely presented group < e0, e1 |  >
                        sage: S1.fat_wedge(4).homology()                                # needs sage.modules
                        {0: 0, 1: Z x Z x Z x Z, 2: Z^6, 3: Z x Z x Z x Z}
                    """
                    from sage.topology.simplicial_set_examples import Point
                    if n == 0:
                        return Point()
                    if n == 1:
                        return self
                    return self.product(*[self]*(n-1)).fat_wedge_as_subset()

                def smash_product(self, *others):
                    """
                    Return the smash product of this simplicial set with ``others``.

                    INPUT:

                    - ``others`` -- one or several simplicial sets

                    EXAMPLES::

                        sage: # needs sage.graphs sage.groups
                        sage: S1 = simplicial_sets.Sphere(1)
                        sage: RP2 = simplicial_sets.RealProjectiveSpace(2)
                        sage: X = S1.smash_product(RP2)
                        sage: X.homology(base_ring=GF(2))                               # needs sage.modules
                        {0: Vector space of dimension 0 over Finite Field of size 2,
                         1: Vector space of dimension 0 over Finite Field of size 2,
                         2: Vector space of dimension 1 over Finite Field of size 2,
                         3: Vector space of dimension 1 over Finite Field of size 2}

                        sage: T = S1.product(S1)                                        # needs sage.graphs sage.groups
                        sage: X = T.smash_product(S1)                                   # needs sage.graphs sage.groups
                        sage: X.homology(reduced=False)                                 # needs sage.graphs sage.groups sage.modules
                        {0: Z, 1: 0, 2: Z x Z, 3: Z}
                    """
                    from sage.topology.simplicial_set_constructions import SmashProductOfSimplicialSets_finite
                    return SmashProductOfSimplicialSets_finite((self,) + others)
